<?php

defined( 'ABSPATH' ) || exit;

/**
 * Filter the Style Manager design assets and
 * add any locally defined ones that we can find in the 'design-assets' folder structure.
 *
 * Please note that the subfolders should be named exactly as the keys used for regular, cloud-based assets.
 * Each design assets config should in a separate .json file.
 */
add_filter( 'style_manager/get_design_assets', function ( $design_assets ) {
	$base_directory = __DIR__ . DIRECTORY_SEPARATOR . 'design-assets';
	if ( ! file_exists( $base_directory ) ) {
		return $design_assets;
	}

	$allowed_design_asset_types = [
		'block_patterns',
		'cloud_fonts',
		'color_palettes',
		'color_palettes_v2',
		'font_palettes',
		'system_fonts',
		'theme_configs',
		'notifications',
	];

	$nested_files               = new \RecursiveIteratorIterator( new \RecursiveDirectoryIterator( $base_directory ) );
	$nested_design_assets_files = new \RegexIterator( $nested_files, '/^.+\.json/i', \RecursiveRegexIterator::GET_MATCH );
	foreach ( $nested_design_assets_files as $path => $file ) {
		// Extract the partial directory/file_name part
		$design_asset_slug = substr(
			$path,
			// Starting position of slug.
			strpos( $path, $base_directory . DIRECTORY_SEPARATOR ) + 1 + strlen( $base_directory ),
			// Subtract ending '.json'.
			- 5
		);
		if ( ! strpos( $design_asset_slug, DIRECTORY_SEPARATOR ) ) {
			continue;
		}
		// Now split the slug by the first directory separator to determine the design asset type.
		$type = substr( $design_asset_slug, 0, strpos( $design_asset_slug, DIRECTORY_SEPARATOR ) );
		if ( empty( $type ) || ! in_array( $type, $allowed_design_asset_types ) ) {
			continue;
		}
		// What remains is the actual slug/name that will be used to identify the design asset.
		$name = substr( $design_asset_slug, strlen( $type . DIRECTORY_SEPARATOR ) );
		if ( empty( $name ) ) {
			$name = wp_unique_id( $type . '_asset' );
		}

		// Load the .json file contents.
		$config = json_decode( file_get_contents( $path ), true );
		if ( empty( $config ) ) {
			// Could not decode the contents.
			continue;
		}

		if ( empty( $design_assets[ $type ] ) ) {
			$design_assets[ $type ] = [];
		}

		// We will not overwrite existing design assets.
		if ( isset( $design_assets[ $type ][ $name ] ) ) {
			continue;
		}

		// Insert the local design asset at the beginning of the list, for easy access.
		$design_assets[ $type ] = [ $name => $config ] + $design_assets[ $type ];
	}

	return $design_assets;
}, 10, 1 );
